#ifndef __MAKET_TCP_H__
#define __MAKET_TCP_H__

#include <vector>
#include <functional>
#include "../util/config.h"

class EthernetMaketClient : public EthernetClient {
protected:
    uint32_t lastActivityTime = 0; // Время последней активности
    const uint16_t timeout = 300;  // Таймаут неактивности в миллисекундах
    uint32_t startConnection = 0;
    const uint16_t connectionTimeout = 350;
    uint32_t stopStartTime = 0;       // Время начала состояния CONNECT_IDLE
    const uint32_t stopTimeout = 300; // Таймаут для закрытия сокета в состоянии CONNECT_IDLE

    std::function<void()> onSendSuccess = []() {};

    bool isNonBlocking = true; // Флаг для определения режима подключения
    ConnectionStatusSimple connectStatus = CONNECT_IDLE;

    uint8_t* dataPtr;
    uint16_t dataSize = 0;
    IPAddress dstIP;
    uint16_t dstPort;
    bool dataWrite();

    static std::vector<EthernetMaketClient*> instances;
    void tick_();
public:
    EthernetMaketClient();
    EthernetMaketClient(uint8_t sock);
    ~EthernetMaketClient();
    ConnectionStatusSimple connectNonBlock(IPAddress ip, uint16_t port);
    void disconnect();
    void close();

    static void tick();
    bool send(uint8_t* data, uint16_t dataSize, IPAddress ip, uint16_t port); 

    bool isConnected() const;

    void setOnSendSuccess(const std::function<void()>& callback);
    void resetOnSendSuccess();
    bool dataWrite(uint8_t* data, uint16_t dataSize, bool override = false);
};

#endif // __MAKET_TCP_H__
